#include "signal.hh"

Bus::Bus( const ZString &pName, int pWidth, int pType, int pNum = 20 ) : 
  width( pWidth ), num( pNum ), type( pType ), count( 0 ), visible( 1 ),
  selected( 0 ), name( pName ), valid( 1 )
{ 
  signals = new char * [ num ];
  for ( int i = 0; i < num; i++ )
    signals[ i ] = new char [ width ];
  makeNameWithoutHierarchy();
} 

void 
Bus::delContents()
{
  for ( int i = 0; i < num; i++ )
    delete [] signals[ i ]; 
  delete [] signals;
}

Bus::Bus( const Bus &b ) : 
  width( b.width ), num( b.num ), type( b.type ), count( b.count ), 
  visible( b.visible ), selected( b.selected ), name( b.name ), 
  valid( b.valid )
{ 
  signals = new char * [ num ];
  for ( int i = 0; i < num; i++ )
  {
    signals[ i ] = new char [ width ];
    strcpy( signals[ i ], b.signals[ i ] );
  }	
}

const Bus & 
Bus::operator=( const Bus &ls )
{
  if ( this != &ls )
  {
    delContents();
    width = ls.width;
    num = ls.num;
    type = ls.type;
    count = ls.count;
    visible = ls.visible;
    selected = ls.visible;
    valid = ls.valid;
    name = ls.name;
    signals = new char * [ num ];
    for ( register int i = 0; i < num; i++ )
    {
      signals[ i ] = new char [ width ];
      strcpy( signals[ i ], ls.signals[ i ] );
    }	
  }
  return *this;
}

void 
Bus::set( int i, unsigned int val )
{
  char retVal[ 33 ];
  toBinStr( val, width, retVal );
  set( i, retVal );
}

const char *
Bus::toStr( int i )
{
  switch ( type )
  {
  case OCT :
    sprintf( str, "%o", toInteger( signals[ i ], 2 ) );
    break;
  case DEC :
    sprintf( str, "%d", toInteger( signals[ i ], 2 ) );
    break;
  case HEX :
    sprintf( str, "%X", toInteger( signals[ i ], 2 ) );
    break;
  default :
    return signals[ i ];
    break;
  }
  return str;
}
