#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "string.hh"
#include "util.hh"

#ifndef _SIGNAL_
#define _SIGNAL_

enum { BIN = 2, OCT = 8, DEC = 10, HEX = 16 };

class Bus
{
public:
  Bus( const ZString &pName, int pWidth, int pType, int pNum = 20 );
  Bus( const Bus &b );
  Bus() : valid( 0 ) {}
  ~Bus() { if ( valid ) delContents(); }
  const Bus & operator=( const Bus &ls );
  int operator==( const Bus &b ) { return name == b.name; }

  int getType() const { return type; }
  void setType( int pType) { type = pType; }
  ZString &getName( int DisplayHierarchy = 1 ) 
    { return ( DisplayHierarchy ) ? name : nameWithoutHierarchy; }
  void setName( const ZString &nName ) 
    { name = nName; makeNameWithoutHierarchy(); }
  int getNum() const { return count; }
  int getWidth() const { return width; }
  void set( const char *c ) { set( count++, c ); }
  void set( int i, const char *c ) { strcpy( signals[ i ], c ); }
  void set( unsigned int val ) { set( count++, val ); }
  void set( int i, unsigned int val );
  int isVisible() const { return visible; }
  int getSelected() const { return selected; }
  void setVisible( int v ) { visible = v; }
  void setSelected( int s ) { selected = s; }

  const char *get( int i ) const { return signals[ i ]; }

  const char *toStr( int i );

private:
  void delContents();
  void makeNameWithoutHierarchy()
  {
    if ( !name.strstr( "." ) )
      nameWithoutHierarchy = name;
    else
    {
      for ( int i = 0; i < name.len(); i++ )
	if ( name[ i ] == '.' )
	{
	  nameWithoutHierarchy = ".../";
	  nameWithoutHierarchy += name( i + 1, name.len() );
	  break;
	}
    }
  }
  char **signals;
  ZString name, nameWithoutHierarchy;
  int num, count, width;
  int type;
  char str[ 33 ];
  int visible, selected, valid;
};
    
#endif
