#include "string.hh"

char *string::EmptyString = ( char * ) "";

//// Constructors ////
string::string( int Length )
{
    ActualLen = BufferLen = Length;
    Buffer = new char [ BufferLen + 1 ];
}

string::string( const char *Rhs )
{
    if ( Rhs == NULL )
        Rhs = EmptyString;
    ActualLen = BufferLen = strlen( Rhs );
    Buffer = new char [ BufferLen + 1 ];
    strcpy( Buffer, Rhs );
}

string::string( char Rhs )
{
    ActualLen = BufferLen = 1;
    Buffer = new char [ 2 ];
    Buffer[ 0 ] = Rhs;
    Buffer[ 1 ] = '\0';
}

string::string( const string &Rhs )
{
    BufferLen = ActualLen = Rhs.ActualLen;
    Buffer = new char [ BufferLen + 1 ];
    strcpy( Buffer, Rhs.Buffer );
}

const string &
string::operator=( const string &Rhs )
{
    if ( this == &Rhs )
        return *this;
    if ( BufferLen < Rhs.ActualLen )
    {
        if ( BufferLen != -1 )
            delete [] Buffer;
        BufferLen = Rhs.ActualLen;    
        Buffer = new char [ BufferLen + 1 ];
    }
    ActualLen = Rhs.ActualLen;
    strcpy( Buffer, Rhs.Buffer );
    return *this;  
}

const string &
string::operator=( const char *Rhs )
{
    int TempLen = strlen( Rhs );
    if ( Rhs == NULL )
    {
        Buffer = EmptyString;
        ActualLen = 0;
    }
    if ( BufferLen < TempLen )
    {
        if ( BufferLen != -1 )
            delete [] Buffer;
        BufferLen = TempLen; 
        Buffer = new char [ BufferLen + 1 ];
    }
    ActualLen = TempLen;
    strcpy( Buffer, Rhs );
    return *this;
}

const string &
string::operator=( char Rhs )
{
    if ( BufferLen < 1 )
    {
        if ( BufferLen != -1 )
            delete [] Buffer;
        BufferLen = 1;    
        Buffer = new char [ 2 ];
    }
    ActualLen = 1;
    Buffer[ 0 ] = Rhs;
    Buffer[ 1 ] = '\0';
    return *this;
}  

const string &
string::operator+=( const string &Rhs )
{
    if ( this != &Rhs )
    {
        if ( BufferLen < ActualLen + Rhs.ActualLen )
	{
            char *Temp = new char [ ActualLen + 1 ];
            strcpy( Temp, Buffer );
            if ( BufferLen != -1 )
	        delete [] Buffer;
	    BufferLen = ActualLen + Rhs.ActualLen;
	    Buffer = new char [ BufferLen + 1 ];
	    strcpy( Buffer, Temp );
	    delete [] Temp;
	}
	ActualLen += Rhs.ActualLen;
        strcat( Buffer, Rhs.Buffer );
    }
    else
        *this *= 2;
    return *this;
}

const string &
string::operator+=( const char *Rhs )
{
    int TempLen = strlen( Rhs );
    if ( BufferLen < ActualLen + TempLen )
    {
        char *Temp = new char [ ActualLen + 1 ];
        strcpy( Temp, Buffer );
        if ( BufferLen != -1 )
            delete [] Buffer;
	BufferLen = ActualLen + TempLen;
	Buffer = new char [ BufferLen + 1 ];
	strcpy( Buffer, Temp );
	delete [] Temp;
    }
    strcat( Buffer, Rhs );
    ActualLen += TempLen;
    return *this;
}

const string &
string::operator+=( char Rhs )
{
    if ( BufferLen < ActualLen + 1 )
    {
        char *Temp = new char [ ActualLen + 1 ];
        strcpy( Temp, Buffer );
        if ( BufferLen != -1 )
            delete [] Buffer;
	BufferLen++;
	Buffer = new char [ BufferLen + 2 ];
	strcpy( Buffer, Temp );
	delete [] Temp;
    }
    Buffer[ ActualLen ] = Rhs;
    Buffer[ ActualLen + 1 ] = '\0';
    ActualLen += 1;
    return *this;
}

string
string::operator+( const string &Rhs ) const
{
    string Temp = *this;
    return Temp += Rhs;
}

string
string::operator+( const char *Rhs ) const
{
    string Temp = *this;
    return Temp += Rhs;
}

string
string::operator+( char Rhs ) const
{
    string Temp = *this;
    return Temp += Rhs;
}

const string &
string::operator*=( int Int )
{
    char *Temp = new char [ ActualLen + 1 ];
    strcpy( Temp, Buffer );
    if ( BufferLen < ActualLen * Int )
    {
        if ( BufferLen != -1 )
	    delete [] Buffer;
	BufferLen = ActualLen * Int;
	Buffer = new char [ BufferLen + 1 ];
	strcpy( Buffer, Temp );
    }
    for ( int i = 0; i < Int - 1; i++ )
        strcat( Buffer, Temp );
    ActualLen *= Int;
    delete [] Temp;
    return *this;
}

string
string::operator*( int Int ) const
{
    string Temp = *this;
    return Temp *= Int;
}

string string::operator()( int Ind1, int Ind2 ) const
{
    string Temp( Ind2 - Ind1 );
    if ( Ind1 >= 0 && Ind2 <= ActualLen )
    {
        int Pos = 0;
        for ( int i = Ind1; i <= Ind2; i++ )
            Temp.Buffer[ Pos++ ] = Buffer[ i ];
        return Temp;
    }
    else
    {
        cerr << "Index out of range in substring operator." << endl;
        return ( Temp = "" );
    }
}

char
string::operator[]( int Index ) const
{
    if ( Index < 0 || Index > ActualLen )
    {
        cerr << "Tried access outside the string's range" << endl;
        return Buffer[ 0 ];
    }
    return Buffer[ Index ];
}

char &
string::operator[]( int Index ) 
{
    static char Empty = '\0';
    if ( Index < 0 || Index > ActualLen )
    {
        cerr << "Tried to access outside the string's range" << endl;
        return Empty;
    }
    return Buffer[ Index ];
} 

ostream &
operator<<( ostream &os, const string &Str )
{
    return os << Str.Buffer;
}

istream &
operator>>( istream &is, string &Str )
{
    return is >> Str.Buffer;    
}
 
int
operator==( const string &Lhs, const string &Rhs )
{
    return strcmp( Lhs.Buffer, Rhs.Buffer ) == 0;
}

int
operator!=( const string &Lhs, const string &Rhs )
{
    return strcmp( Lhs.Buffer, Rhs.Buffer ) != 0;
}

int
operator>=( const string &Lhs, const string &Rhs )
{
    return strcmp( Lhs.Buffer, Rhs.Buffer ) >= 0;
}

int
operator<=( const string &Lhs, const string &Rhs )
{
    return strcmp( Lhs.Buffer, Rhs.Buffer ) <= 0;
}

int
operator>( const string &Lhs, const string &Rhs )
{
    return strcmp( Lhs.Buffer, Rhs.Buffer ) > 0;
}

int
operator<( const string &Lhs, const string &Rhs )
{
    return strcmp( Lhs.Buffer, Rhs.Buffer ) < 0;
}

stringTokenizer::stringTokenizer()
{
  stringVar = NULL;
  valid = 0;
  isFirst = 1;
}

stringTokenizer::stringTokenizer( const string &str )
{
  stringVar = new char [ str.len() + 1 ];
  strcpy( stringVar, str.getBuffer() );
  isFirst = 1;
  valid = 1;
}

void
stringTokenizer::set( const string &str )
{
  stringVar = new char [ str.len() + 1 ];
  strcpy( stringVar, str.getBuffer() );
  isFirst = 1;
  valid = 1;
}

string 
stringTokenizer::next( const string &separator )
{
  string tok;
  char *temp;
  
  if ( !valid )
    return tok = "";

  if ( isFirst )
  {
    temp = strtok( stringVar, separator.getBuffer() );
    isFirst = 0;
  }
  else
  {
    temp = strtok( NULL, separator.getBuffer() );
  }
  return ( temp ) ? tok = temp : tok = "";
}
