#ifndef __APP__
#define __APP__

#define SIG_NAME_HEIGHT 20
enum { BACKGROUND = 0, FONT, SIGNAL, BUS, CURSOR };

#define INTERVAL 300
int timeoutID;
int post_handler( FL_OBJECT *ob, int event, FL_Coord mx, 
		  FL_Coord my, int key, void *xev );
void set_tip( FL_OBJECT *ob, char *s, int enable = 1 );
void do_tips( int id, void *p );

class SignalTypeForm;
class AboutForm;
class ProgramPropertiesForm;

struct IniStruct
{
  IniStruct() 
  {
    MainWindowXPosition = 5;
    MainWindowYPosition = 25;
    MainWindowWidth = 400;
    MainWindowHeight = 410;
    DefaultRadix = 0;
    DisplayHierarchy = 1;
    SignalNameWidth = 120;
    EnableTips = 1;
    BackgroundColorR = 0;
    BackgroundColorG = 0;
    BackgroundColorB = 0;
    FontColorR = 0xffff;
    FontColorG = 0xffff;
    FontColorB = 0xffff;
    SignalColorR = 0xffff;
    SignalColorG = 0xffff;
    SignalColorB = 0;
    BusColorR = 0;
    BusColorG = 0xffff; 
    BusColorB = 0xffff;
    CursorColorR = 0;
    CursorColorG = 0xffff;
    CursorColorB = 0;
  }

  int MainWindowXPosition;
  int MainWindowYPosition;
  int MainWindowHeight;
  int MainWindowWidth;
  int DefaultRadix;
  int DisplayHierarchy;
  int SignalNameWidth;
  int EnableTips;
  int BackgroundColorR; //
  int BackgroundColorG;
  int BackgroundColorB;
  int FontColorR; //
  int FontColorG;
  int FontColorB;
  int SignalColorR; //
  int SignalColorG;
  int SignalColorB;
  int BusColorR; //
  int BusColorG;
  int BusColorB;
  int CursorColorR; //
  int CursorColorG;
  int CursorColorB;
};

class App
{
  friend class ProgramPropertiesForm;

private:
  FL_FORM *mainWindow;
  FL_OBJECT *mainBox;
  FL_OBJECT *openButton, *saveButton, *propButton, *progPropButton;
  FL_OBJECT *cutButton, *copyButton, *pasteButton;
  FL_OBJECT *canvas;
  FL_OBJECT *statusLine;
  FL_OBJECT *fileMenu, *editMenu, *zoomMenu, *helpMenu;
  FL_OBJECT *hScrollBar, *vScrollBar;

  Pixmap busPix, bus2Pix, sigPix;

  IniStruct iniStruct;

  string startFile;

  XColor colors[ 256 ];

  SignalTypeForm *propForm;
  AboutForm *aboutForm;
  ProgramPropertiesForm *progPropForm;

  Bus *sig[ 200 ];
  Bus *tempCutSig, *tempCopySig;
  int numSignals;
  int numSamples;
  int spanLength, lastSpanLength;
  int sigNameWidth;

  int startSample;   // Used in horizontal scrolling
  int lastStartSample;

  int button1, button2, button3;
  int moveDivider;

  int zoomBoxOn;
  int zoomBoxX, zoomBoxY;
  int zoomBoxOrgX, zoomBoxOrgY;

  int cursorPos1, selectedSample;
  int startSignal;
  int selectedSignal;

  Pixmap dBuffer;
  
  int orgPixmapWidth, orgPixmapHeight;

  void makeDblBuff();
  void allocateColors();
  void deleteContents();

  enum { LOCAL, HOME };
  void parseIniFile( int isHome );

  App() {}
  
public:
  App( int argc, char **argv );
  ~App(); 

  void resetScrollBars();
  static void onHScroll( FL_OBJECT *ob, long userData );
  static void onVScroll( FL_OBJECT *ob, long userData );
  static void onOpen( FL_OBJECT *ob, long userData );
  static void onSave( FL_OBJECT *ob, long userData );
  int onOpened( string &fileName );
  void onPrint();
  void onAbout();
  static void onCut( FL_OBJECT *ob, long userData );
  static void onCopy( FL_OBJECT *ob, long userData );
  static void onPaste( FL_OBJECT *ob, long userData );
  static void onProperties( FL_OBJECT *ob, long userData );
  static void onProgProperties( FL_OBJECT *ob, long userData );
  static void onMenu( FL_OBJECT *ob, long userData );
  static void onPaint( FL_OBJECT *ob, long userData )
    { App *app = ( App * ) userData; app->onPaint(); }
  static int onPaint( FL_OBJECT *ob, Window w, int x, int y, XEvent *e, 
		      void *d )
    { App *app = ( App * ) d; app->onPaint(); }
  void onPaint();
  static int onButtonPress( FL_OBJECT *ob, Window w, int x, int y, 
			    XEvent *e, void *d )
    { App *app = ( App * ) d; app->onButtonPress( e ); }
  static int onButtonRelease( FL_OBJECT *ob, Window w, int x, int y, 
			      XEvent *e, void *d )
    { App *app = ( App * ) d; app->onButtonRelease( e ); }
  static int onMouseMove( FL_OBJECT *ob, Window w, int x, int y, 
			  XEvent *e, void *d )
    { App *app = ( App * ) d; app->onMouseMove( e ); }
  void onMouseMove( XEvent *e );
  void onButtonRelease( XEvent *e );
  void onButtonPress( XEvent *e );

  void show( const string &title );
  void run() { fl_do_forms(); }
  void deactivate()
    { fl_deactivate_form( mainWindow ); fl_deactivate_object( canvas ); }
  void activate()
    { fl_activate_form( mainWindow ); fl_activate_object( canvas ); }
  
  void update() { fl_call_object_callback( canvas ); }
  void removeSignal( int index );
  void addSignal( int index );
};

class AboutForm
{
private:
  FL_FORM *mainWindow;
  FL_OBJECT *okButton;
  App *parent;

public:
  AboutForm( App *pApp );
  void show();
  static void onOk( FL_OBJECT *ob, long userData );
};

class SignalTypeForm
{
private:
  FL_FORM *mainWindow;
  FL_OBJECT *typeChoice, *nameText, *okButton, *cancelButton;
  FL_OBJECT *applyButton;
  Bus **bus;
  App *parent;
  int sel;

public:
  SignalTypeForm( App *pApp );
  void show( int i, Bus **pBus );
  int isVisible() const { return fl_form_is_visible( mainWindow ); }
  void setSelected( int i );
  static void onOK( FL_OBJECT *ob, long userData );
  static void onApply( FL_OBJECT *ob, long userData );
  static void onCancel( FL_OBJECT *ob, long userData );
  static void onChoice( FL_OBJECT *ob, long userData ) {}
};

class ColorSelectionForm;

class ProgramPropertiesForm
{
friend class ColorSelectionForm;

private:
  FL_FORM *mainWindow;
  FL_OBJECT *hierarchyButton, *okButton, *cancelButton;
  FL_OBJECT *applyButton, *typeChoice, *tipsButton;
  FL_OBJECT *colorBrowser, *colorButton;
  Bus **bus;
  int validSignals;
  App *parent;
  int sel;
  ColorSelectionForm *colorForm;
  XColor *colors;

public:
  ProgramPropertiesForm( App *pApp );
  void show( Bus **pBus, int valid, XColor *cols );
  int isVisible() const { return fl_form_is_visible( mainWindow ); }
  void setSelected( int i );
  static void onOK( FL_OBJECT *ob, long userData );
  static void onApply( FL_OBJECT *ob, long userData );
  static void onCancel( FL_OBJECT *ob, long userData );
  static void onChoice( FL_OBJECT *ob, long userData ) {}
  static void onColorBrowser( FL_OBJECT *ob, long userData );
  static void onModifyColor( FL_OBJECT *ob, long userData );
  static void onHierarchy( FL_OBJECT *ob, long userData ) {}
  static void onTips( FL_OBJECT *ob, long userData ) {}
};

class ColorSelectionForm
{
private:
  FL_FORM *mainWindow;
  FL_OBJECT *okButton, *cancelButton, *canvas;
  FL_OBJECT *redSlider, *greenSlider, *blueSlider;
  ProgramPropertiesForm *parent;
  int colorIndex;

public:
  ColorSelectionForm( ProgramPropertiesForm *pParent );
  void show( int colIndex );
  int isVisible() const { return fl_form_is_visible( mainWindow ); }
  void update() { fl_call_object_callback( canvas ); }
  void onPaint();
  static void onOK( FL_OBJECT *ob, long userData );
  static void onCancel( FL_OBJECT *ob, long userData );
  static void onRedSlide( FL_OBJECT *ob, long userData );
  static void onGreenSlide( FL_OBJECT *ob, long userData );
  static void onBlueSlide( FL_OBJECT *ob, long userData );

  static void onPaint( FL_OBJECT *ob, long userData )
  { 
    ColorSelectionForm *app = ( ColorSelectionForm * ) userData; 
    app->onPaint(); 
  }
  static int onPaint( FL_OBJECT *ob, Window w, int x, int y, XEvent *e, 
		      void *d )
  { 
    ColorSelectionForm *app = ( ColorSelectionForm * ) d; 
    app->onPaint(); 
  }
};

#endif
